import 'dart:convert';
import 'dart:io';

import 'package:hive/hive.dart';
import 'package:path_provider/path_provider.dart';
import 'package:rdc_flutter_sdk/src/database/db_interface.dart';
import 'package:rdc_flutter_sdk/src/model/hive/rabbit_request_hive.dart';
import 'package:rdc_flutter_sdk/src/model/hive_config.dart';
import 'package:rdc_flutter_sdk/src/model/rabbit_request.dart';

class HiveDBImpl implements HiveDBInterface {
  late HiveConfig _config;

  Future<HiveDBInterface> initHive({String? encryptKey}) async {
    Directory appDocDir = await getApplicationDocumentsDirectory();
    Hive.registerAdapter(RabbitRequestHiveAdapter());
    _config = HiveConfig(
        dbName: 'rdc_flutter_sdk',
        path: appDocDir.path,
        boxNames: {RabbitRequest.boxName},
        cipher: encryptKey != null
            ? HiveAesCipher(base64Url.decode(encryptKey))
            : null);
    return this;
  }

  @override
  Future<Box<T>> hiveBox<T>(String boxName) async {
    return await Hive.openBox<T>(boxName,
        path: _config.path, encryptionCipher: _config.cipher);
  }

  @override
  Future<Map<int, T>> getAll<T>(String boxName) async {
    assert(boxName.isNotEmpty);

    var box = await hiveBox<T>(boxName);

    Map<int, T> results = {};
    for (var key in box.keys) {
      results[key] = box.get(key) as T;
    }
    return results;
  }

  @override
  Future<Map<int, T?>> get<T>(String boxName, int key) async {
    assert(boxName.isNotEmpty);

    var box = await hiveBox<T>(boxName);
    var result = box.get(key);
    return <int, T?>{key: result};
  }

  @override
  Future<void> insert<T>(String boxName, T data) async {
    assert(boxName.isNotEmpty);
    assert(data != null);

    var box = await hiveBox<T>(boxName);
    await box.add(data);
  }

  @override
  Future<void> insertAll<T>(String boxName, List<T> data) async {
    assert(boxName.isNotEmpty);
    assert(data.isNotEmpty);

    var box = await hiveBox<T>(boxName);
    await box.addAll(data);
  }

  @override
  Future<void> upsert<T>(String boxName, int key, T data) async {
    assert(boxName.isNotEmpty);
    assert(data != null);

    var box = await hiveBox<T>(boxName);
    await box.put(key, data);
  }

  @override
  Future<void> delete<T>(String boxName, int key) async {
    assert(boxName.isNotEmpty);

    var box = await hiveBox<T>(boxName);
    await box.delete(key);
  }

  @override
  Future<void> deleteKeys<T>(String boxName, List<int> keys) async {
    assert(boxName.isNotEmpty);

    var box = await hiveBox<T>(boxName);
    await box.deleteAll(keys);
  }

  @override
  Future<void> deleteAll<T>(String boxName) async {
    assert(boxName.isNotEmpty);

    var box = await hiveBox<T>(boxName);
    await box.clear();
  }
}
