import 'dart:async';
import 'dart:collection';
import 'package:flutter/services.dart';

/// This is the [CpuReader] plugin responsible for communication with (Android) platform
/// plugins using asynchronous method calls through [cpu_reader] channel.
class CpuReader {
  static const MethodChannel _channel = MethodChannel('rdc_flutter_sdk');
  static String? _abi;
  static int? _numberOfCores;
  static final HashMap<int, MinMaxFrequency<int>> _minMaxFrequencies =
      HashMap<int, MinMaxFrequency<int>>();

  /// [CpuInfo] stream with set [interval] value as [Duration]
  static Stream<CpuInfo> cpuStream(int interval) async* {
    while (true) {
      await Future.delayed(Duration(milliseconds: interval));
      CpuInfo info = await CpuReader.cpuInfo;
      yield info;
    }
  }

  /// Gets the Android Binary Interface [getAbi] of the device
  static Future<String> getAbi() async {
    try {
      return _abi ?? await _channel.invokeMethod('getAbi');
    } on PlatformException catch (e) {
      throw 'Failed to retrieve abi ${e.code}: ${e.message}';
    }
  }

  // Gets the number of cores
  static Future<int> getNumberOfCores() async {
    try {
      return _numberOfCores ?? await _channel.invokeMethod('getNumberOfCores');
    } on PlatformException catch (e) {
      throw 'Failed to retrieve number of cores ${e.code}: ${e.message}';
    }
  }

  /// Gets the current frequency for the specified [coreNumber]
  static Future<int> getCurrentFrequency(int coreNumber) async {
    try {
      return await _channel
          .invokeMethod('getCurrentFrequency', {"coreNumber": coreNumber});
    } on PlatformException catch (e) {
      throw 'Failed to retrieve current frequency for the core ${e.code}: ${e.message}';
    }
  }

  /// Gets the current min and max frequencies for the specified [coreNumber]
  static Future<MinMaxFrequency<int>?> getMinMaxFrequencies(
      int coreNumber) async {
    try {
      if (_minMaxFrequencies.containsKey(coreNumber)) {
        return _minMaxFrequencies[coreNumber];
      }
      var map = Map<int, int>.from(await _channel
          .invokeMethod('getMinMaxFrequencies', {"coreNumber": coreNumber}));
      var minMax = MinMaxFrequency(map.keys.first, map.values.first);
      _minMaxFrequencies[coreNumber] = minMax;
      return minMax;
    } on PlatformException catch (e) {
      throw 'Failed to retrieve current frequency for the core ${e.code}: ${e.message}';
    }
  }

  /// Retrieves thermal CPU temperature from the device
  static Future<double> getTemperature() async {
    return await _channel.invokeMethod("getCpuTemperature");
  }

  /// This retries the overall information [CpuInfo] of the device CPU.
  static Future<CpuInfo> get cpuInfo async {
    try {
      var cpuInfoJson = await _channel.invokeMethod('getCpuInfo');
      var info = cpuInfoJson;
      var jsonObj = CpuInfo.fromJson(info);
      return jsonObj;
    } on PlatformException catch (e) {
      throw 'Failed to retrieve cpu info ${e.code}: ${e.message}';
    }
  }

  static Future<num> get cpuUsage async {
    try {
      var total = await _channel.invokeMethod('getCpuUsage');
      //print('cpuUsage : $total');
      return total;
    } on PlatformException catch (e) {
      throw 'Failed to retrieve cpu info ${e.code}: ${e.message}';
    }
  }
}

class MinMaxFrequency<T> {
  final T _min;
  final T _max;
  T get min => _min;
  T get max => _max;

  MinMaxFrequency(this._min, this._max);

  /// Converts instance [toJson] format
  Map<String, dynamic> toJson() => {"min": min, "max": max};
}

class CpuInfo {
  int numberOfCores;
  double cpuTemperature;
  String abi;
  Map<int, MinMaxFrequency> minMaxFrequencies = <int, MinMaxFrequency>{};
  Map<int, int> currentFrequencies = <int, int>{};

  CpuInfo(
      {required this.numberOfCores,
      required this.abi,
      required this.minMaxFrequencies,
      required this.currentFrequencies,
      required this.cpuTemperature});

  /// Deserialize the data [json] retrieved from the device through platform specific code
  factory CpuInfo.fromJson(Map<dynamic, dynamic> json) {
    var currentFrequencies = <int, int>{};
    Map.from(json['currentFrequencies']).forEach((key, value) {
      currentFrequencies[key] = value;
    });

    var minMaxFrequencies = <int, MinMaxFrequency<dynamic>>{};
    Map.from(json['minMaxFrequencies']).forEach((key, value) {
      var map = Map.from(value);
      minMaxFrequencies[key] = MinMaxFrequency(map['min'], map['max']);
    });

    return CpuInfo(
        numberOfCores: json['numberOfCores'],
        abi: json['abi'],
        cpuTemperature: json['cpuTemperature'],
        currentFrequencies: currentFrequencies,
        minMaxFrequencies: minMaxFrequencies);
  }

  /// Converts instance to Json format
  Map<String, dynamic> toJson() => {
        "abi": abi,
        "numberOfCores": numberOfCores,
        "cpuTemprature": cpuTemperature,
        "currentFrequencies": currentFrequencies.map(_convert),
        "minMaxFrequencies": minMaxFrequencies.map(_convert)
      };

  /// Helper function to convert Map<Y, T> => Map<String, T>
  MapEntry _convert<T>(int key, T value) {
    return MapEntry(key.toString(), value);
  }
}
